package lupenko.airport.service.impl;

import lombok.extern.slf4j.Slf4j;
import lupenko.airport.dao.AircraftRepository;
import lupenko.airport.domain.Aircraft;
import lupenko.airport.service.AircraftService;
import lupenko.airport.web.dto.AircraftDto;
import lupenko.airport.service.map.AircraftMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;

import java.util.Optional;

@Service
@Slf4j
public class AircraftServiceImpl implements AircraftService {
    
    private final AircraftRepository aircraftRepository;
    private final AircraftMapper aircraftMapper;

    @Autowired
    public AircraftServiceImpl(final AircraftRepository aircraftRepository, final AircraftMapper aircraftMapper) {
        this.aircraftRepository = aircraftRepository;
        this.aircraftMapper = aircraftMapper;
    }

    public Page<AircraftDto> getAllAircraft(final Pageable pageable) {
        log.info("Finding all aircraft {}", pageable);
        return aircraftRepository.findAll(pageable).map(aircraftMapper::toDto);
    }

    @Override
    public AircraftDto createAircraft(final AircraftDto aircraft) {
        log.info("Creating aircraft {}", aircraft);
        Aircraft entity = aircraftMapper.dtoToEntity(aircraft);
        return aircraftMapper.toDto(aircraftRepository.save(entity));
    }

    @Override
    public void deleteAircraft(final Long id) {
        log.info("Deleting aircraft {}", id);
        aircraftRepository.deleteById(id);
    }

    @Override
    public Optional<AircraftDto> getAircraft(final Long id) {
        log.info("Fetching aircraft {}", id);
        return aircraftRepository.findById(id).map(aircraftMapper::toDto);
    }

    @Override
    public Optional<AircraftDto> updateAircraft(final Long id, final AircraftDto dto) {
        log.info("Updating aircraft {}, by {}", id, dto);
        return aircraftRepository.findById(id)
                .map((persistedAircraft) -> {
                    aircraftMapper.copyParametersFromDto(dto, persistedAircraft);
                    return persistedAircraft;
                })
                .map(aircraftRepository::save)
                .map(aircraftMapper::toDto);
    }
}
