package lupenko.airport.service.impl;

import lombok.extern.slf4j.Slf4j;
import lupenko.airport.dao.AirportRepository;
import lupenko.airport.domain.Airport;
import lupenko.airport.service.AirportService;
import lupenko.airport.web.dto.AirportDto;
import lupenko.airport.service.map.AirportMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;

import java.util.Optional;

@Service
@Slf4j
public class AirportServiceImpl implements AirportService {

    private final AirportRepository airportRepository;
    private final AirportMapper airportMapper;

    @Autowired
    public AirportServiceImpl(final AirportRepository airportRepository, final AirportMapper airportMapper) {
        this.airportRepository = airportRepository;
        this.airportMapper = airportMapper;
    }

    public Page<AirportDto> getAirports(final Pageable pageable) {
        log.info("Finding all airports {}", pageable);
        return airportRepository.findAll(pageable).map(airportMapper::toDto);
    }

    @Override
    public AirportDto createAirport(final AirportDto airport) {
        log.info("Creating airport {}", airport);
        Airport entity = airportMapper.dtoToEntity(airport);
        return airportMapper.toDto(airportRepository.save(entity));
    }

    @Override
    public void deleteAirport(final Long id) {
        log.info("Deleting airport {}", id);
        airportRepository.deleteById(id);
    }

    @Override
    public Optional<AirportDto> getAirport(final Long id) {
        log.info("Fetching airport {}", id);
        return airportRepository.findById(id).map(airportMapper::toDto);
    }

    @Override
    public Optional<AirportDto> updateAirport(final Long id, final AirportDto dto) {
        log.info("Updating airport id {}, by {}", id, dto);
        return airportRepository.findById(id)
                .map((persistedAirport) -> {
                    airportMapper.copyParametersFromDto(dto, persistedAirport);
                    return persistedAirport;
                })
                .map(airportRepository::save)
                .map(airportMapper::toDto);
    }
}
