package lupenko.airport.service.impl;

import lombok.extern.slf4j.Slf4j;
import lupenko.airport.dao.FlightRepository;
import lupenko.airport.domain.Flight;
import lupenko.airport.service.FlightService;
import lupenko.airport.web.dto.FlightDto;
import lupenko.airport.service.map.FlightMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;

import java.util.Optional;

@Service
@Slf4j
public class FlightServiceImpl implements FlightService {
    
    private final FlightRepository flightRepository;
    private final FlightMapper flightMapper;

    @Autowired
    public FlightServiceImpl(final FlightRepository flightRepository, final FlightMapper flightMapper) {
        this.flightRepository = flightRepository;
        this.flightMapper = flightMapper;
    }

    @Override
    public Page<FlightDto> getFlights(final Pageable pageable) {
        log.info("Finding all flights {}", pageable);
        return flightRepository.findAll(pageable).map(flightMapper::toDto);
    }

    @Override
    public FlightDto createFlight(final FlightDto dto) {
        log.info("Creating flight {}", dto);
        Flight entity = flightMapper.dtoToEntity(dto);
        return flightMapper.toDto(flightRepository.save(entity));
    }

    @Override
    public void deleteFlight(final Long id) {
        log.info("Deleting flight {}", id);
        flightRepository.deleteById(id);
    }

    @Override
    public Optional<FlightDto> getFlight(final Long id) {
        log.info("Fetching flight {}", id);
        return flightRepository.findById(id).map(flightMapper::toDto);
    }

    @Override
    public Optional<FlightDto> updateFlight(final Long id, final FlightDto dto) {
        log.info("Updating flight id {}, by {}", id, dto);
        return flightRepository.findById(id)
                .map((persistedEntity) -> {
                    flightMapper.copyParametersFromDto(dto, persistedEntity);
                    return persistedEntity;
                })
                .map(flightRepository::save)
                .map(flightMapper::toDto);
    }
}
