package lupenko.airport.service.impl;

import lombok.extern.slf4j.Slf4j;
import lupenko.airport.dao.PassengerRepository;
import lupenko.airport.domain.Passenger;
import lupenko.airport.service.PassengerService;
import lupenko.airport.web.dto.PassengerDto;
import lupenko.airport.service.map.PassengerMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;

import java.util.Optional;

@Service
@Slf4j
public class PassengerServiceImpl implements PassengerService {

    private final PassengerRepository passengerRepository;
    private final PassengerMapper passengerMapper;

    @Autowired
    public PassengerServiceImpl(final PassengerRepository passengerRepository, final PassengerMapper passengerMapper) {
        this.passengerRepository = passengerRepository;
        this.passengerMapper = passengerMapper;
    }

    @Override
    public Page<PassengerDto> getPassengers(final Pageable pageable) {
        log.info("Finding all passengers {}", pageable);
        return passengerRepository.findAll(pageable).map(passengerMapper::toDto);
    }

    @Override
    public PassengerDto createPassenger(final PassengerDto dto) {
        log.info("Creating passenger {}", dto);
        Passenger entity = passengerMapper.dtoToEntity(dto);
        return passengerMapper.toDto(passengerRepository.save(entity));
    }

    @Override
    public void deletePassenger(final Long id) {
        log.info("Deleting passenger {}", id);
        passengerRepository.deleteById(id);
    }

    @Override
    public Optional<PassengerDto> getPassenger(final Long id) {
        log.info("Fetching passenger {}", id);
        return passengerRepository.findById(id).map(passengerMapper::toDto);
    }

    @Override
    public Optional<PassengerDto> updatePassenger(final Long id, final PassengerDto dto) {
        log.info("Updating passenger id {}, by {}", id, dto);
        return passengerRepository.findById(id)
                .map((persistedEntity) -> {
                    passengerMapper.copyParametersFromDto(dto, persistedEntity);
                    return persistedEntity;
                })
                .map(passengerRepository::save)
                .map(passengerMapper::toDto);
    }
}
