package lupenko.airport.service.map;

import lombok.extern.slf4j.Slf4j;
import lupenko.airport.dao.FlightRepository;
import lupenko.airport.dao.PassengerRepository;
import lupenko.airport.domain.Booking;
import lupenko.airport.domain.Flight;
import lupenko.airport.domain.Passenger;
import lupenko.airport.web.dto.BookingDto;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.springframework.beans.factory.annotation.Autowired;

import javax.persistence.EntityNotFoundException;

@Mapper(componentModel = "spring")
@Slf4j
public abstract class BookingMapper {

    @Autowired
    private PassengerRepository passengerRepository;
    @Autowired
    private FlightRepository flightRepository;

    @Mapping(target = "id", ignore = true)
    @Mapping(source = "flightId", target = "flight")
    @Mapping(source = "passengerId", target  = "passenger")
    public abstract Booking dtoToEntity(BookingDto dto);

    @Mapping(target = "flightId", source = "flight.id")
    @Mapping(target = "passengerId", source = "passenger.id")
    public abstract BookingDto toDto(Booking entity);

    public Passenger passengerFromId(Long id) {
        if (id == null) {
            return null;
        }

        log.info("Fetching passenger id {}", id);

        return passengerRepository.findById(id).orElseThrow(EntityNotFoundException::new);
    }

    public Flight flightFromId(Long id) {
        if (id == null) {
            return null;
        }

        log.info("Fetching flight id {}", id);

        return flightRepository.findById(id).orElseThrow(EntityNotFoundException::new);
    }

}
