package lupenko.airport.service.map;

import lombok.extern.slf4j.Slf4j;
import lupenko.airport.dao.AircraftRepository;
import lupenko.airport.dao.AirportRepository;
import lupenko.airport.domain.Aircraft;
import lupenko.airport.domain.Airport;
import lupenko.airport.domain.Flight;
import lupenko.airport.web.dto.FlightDto;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.MappingTarget;
import org.springframework.beans.factory.annotation.Autowired;

import javax.persistence.EntityNotFoundException;

@Mapper(componentModel = "spring")
@Slf4j
public abstract class FlightMapper {

    @Autowired
    private AirportRepository airportRepository;
    @Autowired
    private AircraftRepository aircraftRepository;

    @Mapping(target = "id", ignore = true)
    @Mapping(source = "sourceAirportId", target = "sourceAirport")
    @Mapping(source = "destinationAirportId", target = "destinationAirport")
    @Mapping(source = "aircraftId", target = "aircraft")
    public abstract Flight dtoToEntity(FlightDto dto);

    @Mapping(target = "sourceAirportId", source = "sourceAirport.id")
    @Mapping(target = "destinationAirportId", source = "destinationAirport.id")
    @Mapping(target = "aircraftId", source = "aircraft.id")
    public abstract FlightDto toDto(Flight entity);

    @Mapping(target = "id", ignore = true)
    @Mapping(source = "sourceAirportId", target = "sourceAirport")
    @Mapping(source = "destinationAirportId", target = "destinationAirport")
    @Mapping(source = "aircraftId", target = "aircraft")
    public abstract void copyParametersFromDto(FlightDto dto, @MappingTarget Flight entity);

    public Airport airportFromId(Long id) {
        if (id == null) {
            return null;
        }

        log.info("Fetching airport id {}", id);

        return airportRepository.findById(id).orElseThrow(EntityNotFoundException::new);
    }

    public Aircraft aircraftFromId(Long id) {
        if (id == null) {
            return null;
        }

        log.info("Fetching aircraft id {}", id);

        return aircraftRepository.findById(id).orElseThrow(EntityNotFoundException::new);
    }


}
