package lupenko.airport.web.controller;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lupenko.airport.service.AirportService;
import lupenko.airport.web.dto.AirportDto;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

@RestController
@RequestMapping(value = "/airports", produces = MediaType.APPLICATION_JSON_VALUE)
public class AirportController {

    private final AirportService airportService;

    @Autowired
    public AirportController(final AirportService airportService) {
        this.airportService = airportService;
    }

    @RequestMapping(method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "page", dataType = "integer", paramType = "query",
                    value = "Results page you want to retrieve (0..N)"),
            @ApiImplicitParam(name = "size", dataType = "integer", paramType = "query",
                    value = "Number of records per page."),
            @ApiImplicitParam(name = "sort", allowMultiple = true, dataType = "string", paramType = "query",
                    value = "Sorting criteria in the format: property(,asc|desc). " +
                            "Default sort order is ascending. " +
                            "Multiple sort criteria are supported.")
    })
    @ApiOperation("Gets all airports")
    public @ResponseBody Page<AirportDto> getAirports(@ApiIgnore Pageable pageable) {
        return airportService.getAirports(pageable);
    }

    @RequestMapping(method = RequestMethod.POST, consumes = MediaType.APPLICATION_JSON_VALUE,
            produces = MediaType.APPLICATION_JSON_VALUE)
    @ResponseStatus(HttpStatus.CREATED)
    @ApiOperation("Creates a new airport")
    public @ResponseBody AirportDto createAirport(@RequestBody AirportDto airport) {
        System.out.println(airport);
        return airportService.createAirport(airport);
    }

    @RequestMapping(path = "/{id}", method = RequestMethod.DELETE)
    @ResponseStatus(HttpStatus.NO_CONTENT)
    @ApiOperation("Deletes airport")
    public void deleteAirport(@PathVariable Long id) {
        airportService.deleteAirport(id);
    }

    @RequestMapping(path = "/{id}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    @ApiOperation("Gets airport")
    public ResponseEntity<AirportDto> getAirport(@PathVariable Long id) {
        return ResponseEntity.of(airportService.getAirport(id));
    }

    @RequestMapping(path = "/{id}", method = RequestMethod.PUT, consumes = MediaType.APPLICATION_JSON_VALUE)
    @ApiOperation("Updates airport")
    public ResponseEntity<Void> updateAirport(@PathVariable Long id, @RequestBody AirportDto airport) {
        if (airportService.updateAirport(id, airport).isPresent()) {
            return new ResponseEntity<>(HttpStatus.NO_CONTENT);
        } else {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
    }
}