package lupenko.airport.service.map;

import lupenko.airport.domain.Airport;
import lupenko.airport.web.dto.AirportDto;
import org.junit.jupiter.api.Test;

import java.math.BigDecimal;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNull;

public class AirportMapperTest {

    private static AirportMapperImpl airportMapper = new AirportMapperImpl();

    @Test
    public void dtoToEntity() {
        AirportDto dto = new AirportDto(22L, "CODE", "US", new BigDecimal("2.4"), new BigDecimal("2.7"));

        Airport entity = airportMapper.dtoToEntity(dto);

        assertNull(entity.getId());
        assertEquals(dto.getCountryCode(), entity.getCountryCode());
        assertEquals(dto.getCode(), entity.getCode());
        assertEquals(dto.getDdLongitude(), entity.getDdLongitude());
        assertEquals(dto.getDdLatitude(), entity.getDdLatitude());
    }

    @Test
    public void toDto() {
        Airport entity = new Airport(22L, "CODE", "US", new BigDecimal("2.4"), new BigDecimal("2.7"));

        AirportDto dto = airportMapper.toDto(entity);

        assertEquals(entity.getId(), dto.getId());
        assertEquals(entity.getDdLongitude(), dto.getDdLongitude());
        assertEquals(entity.getDdLatitude(), dto.getDdLatitude());
        assertEquals(entity.getCountryCode(), dto.getCountryCode());
        assertEquals(entity.getCode(), dto.getCode());
    }

    @Test
    public void copyParametersFromDto() {
        Airport entity = new Airport(25L, "CODE", "US", new BigDecimal("2.4"), new BigDecimal("2.7"));
        AirportDto dto = new AirportDto(22L, "CODEDTO", "CZ", new BigDecimal("22.4"), new BigDecimal("12.17"));

        airportMapper.copyParametersFromDto(dto, entity);

        assertEquals(entity.getId(), 25L);
        assertEquals(entity.getCountryCode(), "CZ");
        assertEquals(entity.getDdLatitude(), new BigDecimal("12.17"));
        assertEquals(entity.getDdLongitude(), new BigDecimal("22.4"));
        assertEquals(entity.getCode(), "CODEDTO");
    }
}