package lupenko.airport.service.map;

import lupenko.airport.dao.FlightRepository;
import lupenko.airport.dao.PassengerRepository;
import lupenko.airport.domain.Booking;
import lupenko.airport.domain.Flight;
import lupenko.airport.domain.Passenger;
import lupenko.airport.web.dto.BookingDto;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.springframework.test.util.ReflectionTestUtils;

import java.time.Instant;
import java.util.Optional;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNull;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

public class BookingMapperTest {

    private static final long FLIGHT_ID = 13L;
    private static final long PASSENGER_ID = 37L;
    private static BookingMapperImpl bookingMapper = new BookingMapperImpl();
    private static final Passenger PASSENGER_MOCK = mock(Passenger.class);
    private static final Flight FLIGHT_MOCK = mock(Flight.class);

    @BeforeAll
    public static void mockRepositories() {
        when(PASSENGER_MOCK.getId()).thenReturn(PASSENGER_ID);
        PassengerRepository passengerRepositoryMock = mock(PassengerRepository.class);
        when(passengerRepositoryMock.findById(PASSENGER_ID)).thenReturn(Optional.of(PASSENGER_MOCK));
        ReflectionTestUtils.setField(bookingMapper, "passengerRepository", passengerRepositoryMock);

        when(FLIGHT_MOCK.getId()).thenReturn(FLIGHT_ID);
        FlightRepository flightRepositoryMock = mock(FlightRepository.class);
        when(flightRepositoryMock.findById(FLIGHT_ID)).thenReturn(Optional.of(FLIGHT_MOCK));
        ReflectionTestUtils.setField(bookingMapper, "flightRepository", flightRepositoryMock);
    }

    @Test
    public void dtoToEntity() {
        BookingDto dto = new BookingDto(22L, FLIGHT_ID, PASSENGER_ID, Instant.EPOCH);

        Booking entity = bookingMapper.dtoToEntity(dto);

        assertNull(entity.getId());
        assertEquals(FLIGHT_MOCK, entity.getFlight());
        assertEquals(PASSENGER_MOCK, entity.getPassenger());
        assertEquals(Instant.EPOCH, entity.getCheckout());
    }

    @Test
    public void toDto() {
        Booking entity = new Booking(22L, FLIGHT_MOCK, PASSENGER_MOCK, Instant.EPOCH);

        BookingDto dto = bookingMapper.toDto(entity);

        assertEquals(22L, dto.getId());
        assertEquals(FLIGHT_ID, dto.getFlightId());
        assertEquals(PASSENGER_ID, dto.getPassengerId());
        assertEquals(Instant.EPOCH, dto.getCheckout());
    }

}