package lupenko.airport.service.map;

import lupenko.airport.domain.Passenger;
import lupenko.airport.web.dto.PassengerDto;
import org.junit.jupiter.api.Test;

import java.time.LocalDate;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNull;

public class PassengerMapperTest {

    private static PassengerMapperImpl passengerMapper = new PassengerMapperImpl();

    @Test
    public void dtoToEntity() {
        PassengerDto dto = new PassengerDto("Name", "LastName", "CZ", LocalDate.MIN, "ASD123");

        Passenger entity = passengerMapper.dtoToEntity(dto);

        assertNull(entity.getId());
        assertEquals(dto.getLastName(), entity.getLastName());
        assertEquals(dto.getFirstName(), entity.getFirstName());
        assertEquals(dto.getPassport(), entity.getPassport());
        assertEquals(dto.getBirthDate(), entity.getBirthDate());
        assertEquals(dto.getAddress(), entity.getAddress());
    }

    @Test
    public void toDto() {
        Passenger entity = new Passenger(37L, "Name", "LastName", "CZ", LocalDate.now(), "ASD123");

        PassengerDto dto = passengerMapper.toDto(entity);

        assertEquals(entity.getId(), dto.getId());
        assertEquals(entity.getLastName(), dto.getLastName());
        assertEquals(entity.getFirstName(), dto.getFirstName());
        assertEquals(entity.getPassport(), dto.getPassport());
        assertEquals(entity.getBirthDate(), dto.getBirthDate());
        assertEquals(entity.getAddress(), dto.getAddress());
    }

    @Test
    public void copyParametersFromDto() {
        Passenger entity = new Passenger(37L, "Name", "LastName", "CZ", LocalDate.now(), "ASD123");
        PassengerDto dto = new PassengerDto(13L, "NameUPD", "LastNameUPD", "US", LocalDate.MIN, "ASD12UPD");

        passengerMapper.copyParametersFromDto(dto, entity);

        assertEquals(entity.getId(), 37L);
        assertEquals(dto.getLastName(), entity.getLastName());
        assertEquals(dto.getFirstName(), entity.getFirstName());
        assertEquals(dto.getPassport(), entity.getPassport());
        assertEquals(dto.getBirthDate(), entity.getBirthDate());
        assertEquals(dto.getAddress(), entity.getAddress());
    }
}